# -*- coding: utf-8 -*-

set :application, "xec"

# see http://github.com/guides/deploying-with-capistrano
default_run_options[:pty] = true

set :application, 'ec_admin'
set :repository,  "svn+ssh://xxxxxxxxxxxxxxxxxxxxxxxxxx/#{$ecr_application_name_in_repository}"
set :user, "xec"

# ssh_options[:user] = 'XXXXXXXXXX'
# ssh_options[:password] = 'XXXXXXXXXXX'
# ssh_options[:forward_agent] = true
# default_run_options[:pty] = true
set :use_sudo, true

set :keep_releases, 3

if zabbix_host = ENV['ZABBIX_HOST']
  role :app, zabbix_host
  role :web, zabbix_host
  role :db,  zabbix_host, :primary => true
end



desc "Show usage"
task :usage do
  puts <<-EOS
    1. Setup your Zabbix server
      see http://www.zabbix.com/ , http://www.zabbix.jp/
    2. Create a user for registering host to your Zabbix server.
       And set Language English(GB) to the user.
    3. Start your zabbix target host on EC2
    4. Check your target host IP
      ec2-metadata is very useful.
      see http://developer.amazonwebservices.com/connect/entry.jspa?externalID=1825
    5. Check templates for your target host
      $ cap -f config/deploy-zabbix.rb zabbix_server:templates
    6. Check groups for your target host
      $ cap -f config/deploy-zabbix.rb zabbix_server:groups
    7. Deplay zabbix-register to your target host
      $ ZABBIX_HOST=174.129.197.111 \\
        ZABBIX_GROUPS="Linux servers,ecrider" \\
        ZABBIX_TEMPLATES=Template_ECRIDER_LB_www \\
        cap -f config/deploy-zabbix.rb deploy
    8. Register your target host to your Zabbix server
      $ ZABBIX_HOST=174.129.197.111 \\
        cap -f config/deploy-zabbix.rb deploy:start
    8. Restart zabbix-agent on your target host
      $ ZABBIX_HOST=174.129.197.111 \\
        cap -f config/deploy-zabbix.rb zabbix_agent:restart
    9. Setup actions for your target host on your Zabbix server.
  EOS
end

desc "Show usage"
task(:help){ usage }

def load_config
  require 'yaml'
  require File.join(File.dirname(__FILE__), "zabbix/etc/zabbix-register/zabbix_frontend_client")
  YAML.load_file(File.join(File.dirname(__FILE__), 'deploy-zabbix.yml'))
end

def zabbix_client
  config = load_config
  server = config.delete('server')
  clinet = ZabbixFrontendClient.new(
    server['url'], server['username'], server['password'], :debug => config['debug'])
end

def upload_file(*args)
  if block_given?
    dest_path, owner_group, permissions = *args
    filename = File.basename(dest_path)
    src = yield
    put(src, "~/#{filename}", :via => :scp)
  else
    src_path, dest_path, owner_group, permissions = *args
    filename = File.basename(dest_path)
    put(IO.read(src_path), "~/#{filename}", :via => :scp)
  end
  cmd = "#{sudo} mv ~/#{filename} #{dest_path}"
  cmd << " && #{sudo} chown #{owner_group} #{dest_path}" if owner_group
  cmd << " && #{sudo} chmod #{permissions} #{dest_path}" if permissions
  run(cmd)
end

def check_zabbix_host
  # ENV['ZABBIX_HOST'] でデプロイ先のホストを指定します。
  zabbix_host = ENV['ZABBIX_HOST']
  raise "ERROR! You must set ZABBIX_HOST=xxx.xxx.xxx.xxx. Try 'Try 'cap -f config/deploy-zabbix.rb usage'" unless zabbix_host
  zabbix_host
end


def deploy_as(zabbix_host, visibility = 'local')
  # 引数visibilityは、ERB中で使用されている変数です。
  zabbix_host = check_zabbix_host
  
  raise "ERROR! You must set ZABBIX_TEMPLATES=Template_Linux. Try 'cap -f config/deploy-zabbix.rb zabbix_server:templates'" unless ENV['ZABBIX_TEMPLATES']
  zabbix_templates = ENV['ZABBIX_TEMPLATES'].split(/\,/).map{|s| s.strip}

  require 'erb'
  require 'yaml'

  run "#{sudo} mkdir -p /etc/zabbix-register"
  base_path = File.join(File.dirname(__FILE__), 'zabbix')
  files = Dir[File.join(base_path, '**/*.{sh,rb,yml,erb}')]
  files.reject!{|f| f =~ /zabbix-register\.yml\.erb$/} # 別でアップロードするので除外
  dirs = files.map{|f| File.dirname(f).sub(/^#{base_path}/, '')}.uniq
  dirs.each do |dir_path|
    run "#{sudo} mkdir -p #{dir_path}"
  end
  files.each do |src_path|
    server_path = src_path.sub(/^#{base_path}/, '')
    upload_file(src_path, server_path, 'root:root')
  end
  sh_filename = "zabbix-register.sh"
  sh_path = "/etc/init.d/#{sh_filename}"
  rb_path = "/etc/zabbix-register/zabbix-register.rb"
  run "#{sudo} chmod 755 #{sh_path}"
  run "#{sudo} chmod 755 #{rb_path}"
  # /etc/rc#{x},d/ へsymlinkを設定
  symlinks = 
    [0,1,6  ].map{|idx| "#{sudo} ln -nfs #{sh_path} /etc/rc#{idx}.d/K40#{sh_filename}"} +
    [2,3,4,5].map{|idx| "#{sudo} ln -nfs #{sh_path} /etc/rc#{idx}.d/S40#{sh_filename}"}
  run(symlinks.join(" && "))

  upload_file("/etc/zabbix-register/zabbix-register.yml", "root:root") do
    config = load_config
    server = config.delete('server') # ERB中で使用されるローカル変数です。
    clinet = ZabbixFrontendClient.new(
      server['url'], server['username'], server['password'], :debug => config['debug'])
    clinet.login
    clinet.index
    all_groups = clinet.groups
    all_templates = clinet.templates
    #
    zabbix_groups = (ENV['ZABBIX_GROUPS'] || "Linux servers").split(/\,/).map{|s| s.strip}
    groups_work = all_groups.inject({}){|dest, (key, val)| dest[val] = key; dest}
    groups = Hash[*zabbix_groups.map{|t| [groups_work[t], t]}.flatten]
    #
    templates_work = all_templates.inject({}){|dest, (key, val)| dest[val] = key; dest}
    templates = Hash[*zabbix_templates.map{|t| [templates_work[t], t]}.flatten]
    # zabbix-register.yml.erb
    path = File.join(base_path, "/etc/zabbix-register/zabbix-register.yml.erb")
    filename = "zabbix-register.yml"
    server_path = "/etc/zabbix-register/#{filename}"
    ERB.new(IO.read(path), nil, '-').result(binding)
  end
end


desc "接続テスト用のタスク"
task :hello, :roles => [:app, :web, :db] do
  run "echo HelloWorld! $HOSTNAME"
end

namespace :startup do
  desc "Start up with using local IP/hostname"
  task :local do
    deploy.local
    deploy.show_config
    zabbix_agent.update_config
    zabbix_agent.restart
    deploy.start
  end

  desc "Start up with using public IP/hostname"
  task :public do
    deploy.public
    deploy.show_config
    zabbix_agent.update_config
    zabbix_agent.restart
    deploy.start
  end
  
  desc "Start up with using local IP/hostname"
  task(:default){ local }
end

namespace :deploy do
  %w(local public).each do |visibility|
    desc "deploy zabbix-register with using #{visibility} hostname/IP address"
    task visibility.to_sym do
      deploy_as(zabbix_host, visibility)
    end
  end
  
  task(:default){ local }

  %w(start stop).each do |cmd|
    desc "#{cmd} zabbix-registration"
    task cmd.to_sym do
      check_zabbix_host
      run "#{sudo} /etc/init.d/zabbix-register.sh #{cmd}"
    end
  end
  
  desc "Show zabbix-register configuration"
  task :show_config do
    check_zabbix_host
    run "#{sudo} /etc/zabbix-register/zabbix-register.rb -S"
  end
end


namespace :rubygems do
  desc "show rubygems version"
  task :version do
    run "ruby -v"
    run "gem -v"
  end

  desc "Download and setup Rubygems 1.3.5"
  task :setup do
    check_zabbix_host
    run "#{sudo} apt-get remove rubygems1.8 -y"
    run "#{sudo} apt-get install build-essential ri rdoc ruby-dev -y"
    run(
      "wget http://gemcutter-production.s3.amazonaws.com/rubygems/rubygems-1.3.5.tgz" <<
      " && tar zxf rubygems-1.3.5.tgz" <<
      " && cd rubygems-1.3.5/" <<
      " && sudo ruby setup.rb" <<
      " && sudo ln -s /usr/bin/gem1.8 /usr/bin/gem")
  end


  desc "Download and setup Rubygems 1.3.5 for ubuntu 8.04"
  task :setup_ubuntu_804 do
    check_zabbix_host
    run "#{sudo} apt-get remove rubygems -y"
    run "#{sudo} apt-get install build-essential ri rdoc ruby1.8-dev -y"
    run(
      "wget http://gemcutter-production.s3.amazonaws.com/rubygems/rubygems-1.3.5.tgz" <<
      " && tar zxf rubygems-1.3.5.tgz" <<
      " && cd rubygems-1.3.5/" <<
      " && sudo ruby setup.rb" <<
      " && sudo ln -s /usr/bin/gem1.8 /usr/bin/gem")
  end

end


namespace :gems do
  
  namespace :install do
    desc "only gems"
    task :only_gems do
      check_zabbix_host
      run "#{sudo} gem install nokogiri httpclient --no-ri --no-rdoc"
    end
    
    desc "install gems used by zabbix-register to Ubuntu 8.10"
    task :ubuntu do
      check_zabbix_host
      run "#{sudo} apt-get install build-essential ri rdoc ruby-dev libxml2 libxml2-dev libxslt1-dev -y"
      only_gems
    end
    
    desc "install gems used by zabbix-register to Ubuntu 8.04"
    task :ubuntu_804 do
      check_zabbix_host
      run "#{sudo} apt-get install build-essential ri rdoc ruby1.8-dev libxml2 libxml2-dev libxslt1-dev -y"
      only_gems
    end
    
    task(:default){ ubuntu_08_10 }
  end

end

namespace :zabbix_agent do
  namespace :install do
    desc "install zabbix-agent to Ubuntu by apt-get"
    task :ubuntu do
      check_zabbix_host
      run "#{sudo} apt-get install zabbix-agent -y"
    end
    
    desc "install zabbix-agent to Ubuntu by apt-get"
    task(:default){ ubuntu_08_10 }
  end
  

  desc "Update Config"
  task :update_config do
    check_zabbix_host
    run "#{sudo} /etc/zabbix-register/zabbix-register.rb -U"
  end

  %w(start stop restart).each do |cmd|
    desc "#{cmd} zabbix-agent"
    task cmd.to_sym do
      check_zabbix_host
      run "#{sudo} mkdir -p /var/run/zabbix-agent"
      run "#{sudo} /etc/init.d/zabbix-agent #{cmd}"
    end
  end
end

namespace :zabbix_server do

  desc "Show Zabbix server templates"
  task :templates do
    clinet = zabbix_client
    clinet.login
    clinet.index
    templates = clinet.templates
    templates.keys.sort_by{|key| key.to_i}.each do |template_id|
      puts "#{template_id}: #{templates[template_id]}"
    end
  end

  desc "Show Zabbix server groups"
  task :groups do
    clinet = zabbix_client
    clinet.login
    clinet.index
    groups = clinet.groups
    groups.keys.sort_by{|key| key.to_i}.each do |group_id|
      puts "#{group_id}: #{groups[group_id]}"
    end
  end
end


