# -*- coding: utf-8 -*-
require 'mongrel_cluster/recipes'

set :application, $ecr_application_name

set :repository,  "svn+ssh://xxxxxxxxxxxxxxxx/#{$ecr_application_name_in_repository}"

set :user, "xec"

# ssh_options[:user] = 'XXXXX'
# ssh_options[:password] = 'XXXXX'
# ssh_options[:forward_agent] = true
# default_run_options[:pty] = true
set :use_sudo, false

# If you aren't deploying to /u/apps/#{application} on the target
# servers (which is the default), you can specify the actual location
# via the :deploy_to variable:
# set :deploy_to, "/var/www/#{set :deploy_via, :remote_cacheapplication}"

# set :deploy_via, :local_cache
set :deploy_via, :remote_cache
# set :deploy_via, :copy
set :deploy_base, '/home/xec/capistrano'
set :deploy_to, File.join('/home/xec/capistrano', application)


# If you aren't using Subversion to manage your source code, specify
# your SCM below:
# set :scm, :subversion
set :scm, :subversion

# set :scm_prefer_prompt, true # パスワードのログを出す
set :scm_prefer_prompt, false # パスワードのログを出さない
set :scm_user, "XXXXX"
set :scm_password, 'XXXXX'
# set :scm_user do Capistrano::CLI.ui.ask('SVN User: ') end
# set :scm_password do Capistrano::CLI.password_prompt('SVN Password: ') end

# cap deploy:cleanup を実行する際に、キープするリリースの数
# この設定で問題がなければそのうちこの値を2にしたいです。
set :keep_releases, 3


def run_locally(cmd)
  logger.trace "executing locally: #{cmd.inspect}" if logger
  # `#{cmd}`
  result = `#{cmd}`
  # system(cmd)
  logger.trace "executed locally : #{result}" if logger
  result
end


_cset(:run_method)        { :run_locally }
def run(*args, &block) 
  result = run_locally(args.first)
  yield({}, :out, result) if block_given?
  result
end

class MultiLoggerOutput
  def initialize(*dests)
    @dests = dests
  end
  def puts(msg)
    @dests.each{|dest| dest.puts(msg) rescue nil}
    nil
  end
end

deploy_log_file = File.open(File.join(File.dirname(__FILE__), '..', '..', 'log', 'deploy.log'), 'a')
deploy_log_file.puts("\n")
deploy_log_file.puts("-" * 80)
deploy_log_file.puts("#{$ecr_application_name} #{$0} #{ARGV.join(' ')}  - started at #{Time.now.strftime('%Y/%m/%d %H:%M:%S')}")
logger.instance_variable_set(:@device, MultiLoggerOutput.new(STDOUT, deploy_log_file))



target_host = ENV['ECR_CAP_HOST'] || "localhost"
SETTING_NAME = (target_host == "localhost") ? ENV['HOSTNAME'] : target_host

role :app, target_host
role :web, target_host
role :db,  target_host, :primary => true


def setting_for_host
  setting_name = SETTING_NAME || "localhost"
  if setting_name
    setting_file = File.join(File.dirname(__FILE__), setting_name)
    @logger.info("loading #{setting_file}...")
    if File.exist?(setting_file)
      @logger.info("setting file #{setting_file} exist!")
      command_text = instance_eval('"%s"' % IO.read(setting_file))
      return command_text.split(/$/).
        map{|line| line.strip}.select{|line| !line.empty?}.reject{|line| /^\s*\#/ =~ line}.join(' && ')
    end
    @logger.info("no setting file for #{setting_file}")
  end
  @logger.info("no setting")
  return nil
end



def run_with_env(text, options = nil)
  options = {:add_settings => true}.update(options || {})
  commands = []
  # commands << setting_for_host if options[:add_settings]
  commands << text.split(/$/).map{|line| line.strip}.select{|line| !line.empty?}.join(' ')
  run(commands.select{|line| !!line}.map{|line| line.to_s.strip}.select{|line| !line.empty?}.join(' && '))
end


def rm_rf_and_ln_nfs(hashs)
  commands = hashs.map{|hash|
    hash.map{ |src, dest| "rm -rf #{dest} && ln -nfs #{src} #{dest}"}.join(' && ')
  }.join(' && ')
  run_with_env(commands)
end

def evacuate_and_restore(base_dir, *sources)
  options = sources.last.is_a?(Hash) ? sources.pop : {}
  options = {:delete_evacuation_dir => false}.update(options)
  sources.each do |source|
    dest = File.join(base_dir, "tmp/deploy_evaluation", source)
    dest_dir = File.dirname(dest)
    src = File.join(base_dir, source)
    run("mkdir -p #{dest_dir}")
    run("rm -rf #{dest}") # mvする前にtmpに同名のディレクトリがないように消しておきます。
    run("mv #{src} #{dest_dir}")
  end
  yield
  sources.each do |source|
    dest = File.join(base_dir, "tmp/deploy_evaluation", source)
    dest_dir = File.dirname(dest)
    src = File.join(base_dir, source)
    src_dir = File.dirname(src)
    run("rm -rf #{src}") # cp -Rの前に移動先を消しておきます。
    run("mv #{dest} #{src_dir}")
    run("rm -rf #{dest}") # 一応消しておきます。
  end
  run("rm -rf #{base_dir}/tmp/deploy_evaluation") if options[:delete_evacuation_dir]
end


namespace :db do
  desc "Create database yaml in shared path" 
  task :default do
    run("mkdir -p #{shared_path}/config")
    run(<<-"EOS")
      cp config/database.yml.example.server #{shared_path}/config/database.yml;
      echo "*************************************************************
!!!!! IIMPORTANT !!!!!

You MUST CHANGE PASSWORD #{shared_path}/config/database.yaml

*************************************************************"
    EOS
  end

  desc "Make symlink for database yaml" 
  task :symlink do
    run "ln -nfs #{shared_path}/config/database.yml #{release_path}/config/database.yml" 
  end
end


namespace :selenium do
  desc "Create selenium-on-rails config yaml in shared path" 
  task :default do
    run "mkdir -p #{shared_path}/selenium"
  end

  desc "Make symlink for selenium-on-rails config yaml" 
  task :symlink do
    run <<-"EOS"
      if [ -s #{shared_path}/selenium/config.yml ]; then
        ln -nfs #{shared_path}/selenium/config.yml #{release_path}/vendor/plugins/selenium-on-rails/config.yml;
      fi
    EOS
  end
end

namespace :robots do
  desc "Make symlink for robots.txt" 
  task :symlink do
    run <<-"EOS"
      if [ -s #{shared_path}/robots.txt ]; then
        ln -nfs #{shared_path}/robots.txt #{release_path}/public/robots.txt;
      fi
    EOS
  end
end

namespace :amazon do
  desc "download cached files from CloudFront to local public"
  task :download_cache do
    require 'aws/s3'
    if $ecr_application_name == 'ec_front' && EcRider.config.cloud_front_url
      AWS::S3::Base.establish_connection!(
        :access_key_id     => EcRider.config.s3_access_key_id,
        :secret_access_key => EcRider.config.s3_secret_access_key)
      bucket_for_public =  AWS::S3::Bucket.find(EcRider.config.s3_bucket_name)
      bucket_for_public.objects do |obj|
        
      end
      
    end
  end
end


namespace :cms do
  desc "Setup versioned dir"
  task :setup_versioned_dir do
    run_with_env(<<-"EOS")
      if [ -d $ECR_CMS_GIT_DIR ]; then
        echo 'cms_git_dir directory exist! - $ECR_CMS_GIT_DIR'; true;
      else
        mkdir -p $ECR_CMS_GIT_DIR &&
        mkdir -p $ECR_CMS_GIT_DIR/app/views &&
        touch    $ECR_CMS_GIT_DIR/app/views/.gitignore &&
        echo "/error_mailer"      >> $ECR_CMS_GIT_DIR/app/views/.gitignore &&
        echo "/front_site_mailer" >> $ECR_CMS_GIT_DIR/app/views/.gitignore &&
        echo "/sitemap"           >> $ECR_CMS_GIT_DIR/app/views/.gitignore &&
        mkdir -p $ECR_CMS_GIT_DIR/app/views/cms &&
        touch    $ECR_CMS_GIT_DIR/app/views/cms/.gitignore &&
        mkdir -p $ECR_CMS_GIT_DIR/app/views/layouts &&
        touch    $ECR_CMS_GIT_DIR/app/views/layouts/.gitignore &&
        echo "/application*.html.erb"                >> $ECR_CMS_GIT_DIR/app/views/layouts/.gitignore &&
        echo "/cms*.html.erb"                        >> $ECR_CMS_GIT_DIR/app/views/layouts/.gitignore &&
        echo "/_double_submitt_prevention*.html.erb" >> $ECR_CMS_GIT_DIR/app/views/layouts/.gitignore &&
        mkdir -p $ECR_CMS_GIT_DIR/public &&
        mkdir -p $ECR_CMS_GIT_DIR/public/cms &&
        touch    $ECR_CMS_GIT_DIR/public/cms/.gitignore &&
        mkdir -p $ECR_CMS_GIT_DIR/public/images &&
        touch    $ECR_CMS_GIT_DIR/public/images/.gitignore &&
        mkdir -p $ECR_CMS_GIT_DIR/public/javascripts &&
        touch    $ECR_CMS_GIT_DIR/public/javascripts/.gitignore &&
        mkdir -p $ECR_CMS_GIT_DIR/public/stylesheets &&
        touch    $ECR_CMS_GIT_DIR/public/stylesheets/.gitignore &&
        mkdir -p $ECR_CMS_GIT_DIR/public/ec_rider/product_images &&
        touch    $ECR_CMS_GIT_DIR/public/ec_rider/product_images/.gitignore &&
        mkdir -p $ECR_CMS_GIT_DIR/public/banner_images && 
        touch    $ECR_CMS_GIT_DIR/public/banner_images/.gitignore &&
        echo "/tmp_inquiry_images"  >  $ECR_CMS_GIT_DIR/public/.gitignore &&
        echo "/ecrider_javascripts" >> $ECR_CMS_GIT_DIR/public/.gitignore &&
        cd $ECR_CMS_GIT_DIR &&
        git init &&
        git add . &&
        git commit -a -m 'initial commit'; true;
      fi
    EOS
  end
end

namespace :apache2 do
  desc "mkdir in capistrano for apache2"
  task :mkdir_log_for_apache2 do
    # 以下のようなログを出力しようとします。
    # /home/xec/capistrano/log/apache2/ec_admin_error.log
    run("mkdir -p #{deploy_to}/../log/apache2")
  end
end
after "deploy:setup", "apache2:mkdir_log_for_apache2"



desc "接続テスト用のタスク"
task :hello, :roles => [:app, :web, :db] do
  run "echo HelloWorld! $HOSTNAME"
end

desc "confirm ENV"
task :confirm_env do
  run "env | sort"
end

before "deploy:setup", :confirm_env
before "deploy:setup", :db
before "deploy:setup", :selenium
before "deploy:setup", "robots:symlink"
after "deploy:setup", "cms:setup_versioned_dir"

before "deploy:update_code", :confirm_env
after "deploy:update_code", "db:symlink"
after "deploy:update_code", "selenium:symlink"

namespace :deploy do
  task :update do
    transaction do
      update_code
      symlink
    end
    begin
      cleanup 
    rescue
      logger.debug("failed to cleanup old releases, but deploy:update success!!")
    end
  end

  task :show_releases do
    logger.info "releases: #{releases.inspect}" if logger
  end

end



namespace :backup do
  
  require 'yaml'
  require 'erb'

  desc "Create a backup"
  task :create do
    backup_projects = %w(ec_admin ec_front ec_test)
    backup_name = Time.now.strftime('%Y%m%d%H%M%S')
    backup_dir = File.expand_path("#{deploy_to}/../backups/#{backup_name}")
    # capistrano/backups/YYYYmmddHHMMSS/ec_*/shared の作成
    run(backup_projects.map{|proj| "mkdir -p #{backup_dir}/#{proj}/shared"}.join(" && "))
    #
    config = YAML.load(ERB.new(IO.read(File.join(shared_path, 'config/database.yml'))).result)
    c = config[ENV['RAILS_ENV'] || 'production']
    mysqldump_opts = "-u #{c['username']}"
    mysqldump_opts << ' -p' if c['password']
    mysqldump_opts << " -h #{c['host']}"
    mysqldump_opts << " --socket=#{c['socket']}" if c['socket'] 
    mysqldump_opts << " " << ENV['MYSQL_OPTS'] if ENV['MYSQL_OPTS']
    mysqldump_opts << " " << ENV['MYSQLDUMP_OPTS'] if ENV['MYSQLDUMP_OPTS']
    cmd = "mysqldump #{mysqldump_opts} #{c['database']} > #{backup_dir}/#{c['database']}.dump.sql"
    logger.info(cmd) if logger
    run(cmd)
    # versioned
    cmd = "cp -R $ECR_CMS_GIT_DIR #{backup_dir}/versioned"
    logger.info(cmd) if logger
    run_with_env(cmd)
    # each shared
    %w(ec_admin ec_front ec_test).each do |proj|
      src_shared = File.expand_path("#{deploy_to}/../#{proj}/shared")
      Dir.chdir(src_shared) do
        targets = Dir.glob('*') - %w(cached-copy log pids) # pidsなどはバックアップ対象外
        cmd = targets.map{|t| "cp -R #{t} #{backup_dir}/#{proj}/shared" }.join(" && ")
        logger.info(cmd) if logger
        run(cmd)
      end
    end
    # archive in a tar ball
    Dir.chdir(File.expand_path("#{deploy_to}/../backups")) do
      run("tar zcf xec_backup-#{backup_name}.tar.gz #{backup_name}")
    end
  end

  desc "Restore from a backup"
  task :restore do
    tar_ball_path = ENV['BACKUP_TAR_BALL']
    backups_dir = File.expand_path("#{deploy_to}/../backups")
    run("mkdir -p #{backups_dir}")
    if tar_ball_path && !tar_ball_path.empty?
      tar_ball_path = File.expand_path(tar_ball_path)
      FileUtils.cp(tar_ball_path, backups_dir) unless File.dirname(tar_ball_path) == backups_dir
      dirname = nil
      Dir.chdir(backups_dir) do
        files = Dir.glob("*")
        run("tar zxf #{File.basename(tar_ball_path)}")
        dirname = (Dir.glob("*") - files).first
      end
      ENV['BACKUP_DIR'] = File.join(backups_dir, dirname)
    end
    raise "You MUST set BACKUP_TAR_BALL or BACKUP_DIR" unless ENV['BACKUP_DIR']
    #
    backup_dir = File.expand_path(ENV['BACKUP_DIR'])
    backup_projects = %w(ec_admin ec_front ec_test)
    backup_projects.each do |proj|
      proj_shared_path = File.expand_path(File.join(backup_dir, proj, 'shared'))
      raise "#{proj_shared_path} doesn't exist." unless File.exist?(proj_shared_path)
      raise "#{proj_shared_path} isn't a directory." unless File.directory?(proj_shared_path)
    end
    versioned_path = File.expand_path(File.join(backup_dir, 'versioned'))
    raise "#{versioned_path} doesn't exist." unless File.exist?(versioned_path)
    raise "#{versioned_path} isn't a directory." unless File.directory?(versioned_path)
    # MySQL
    config = YAML.load(ERB.new(IO.read(File.join(shared_path, 'config/database.yml'))).result)
    c = config[ENV['RAILS_ENV'] || 'production']
    mysql_opts = "-u #{c['username']}"
    mysql_opts << ' -p' if c['password']
    mysql_opts << " -h #{c['host']}"
    mysql_opts << " --socket=#{c['socket']}" if c['socket'] 
    mysql_opts << " " << ENV['MYSQL_OPTS'] if ENV['MYSQL_OPTS']
    Dir.chdir(backup_dir) do
      dumps = Dir.glob("*.dump.sql")
      raise "No mysql dump file found." if dumps.empty?
      dumps.each do |dump|
        run("mysql #{mysql_opts} #{c['database']} < #{dump}")
      end
    end
    # versioned
    cmd = "rm -rf $ECR_CMS_GIT_DIR && cp -R #{backup_dir}/versioned $ECR_CMS_GIT_DIR"
    logger.info(cmd) if logger
    run_with_env(cmd)
    # each shared
    backup_projects.each do |proj|
      dest_shared = File.expand_path("#{deploy_to}/../#{proj}/shared")
      Dir.chdir("#{backup_dir}/#{proj}/shared") do
        targets = Dir.glob('*') - %w(cached-copy log pids) # pidsなどはバックアップ対象外
        cmd = targets.map{|target| 
          dest = File.join(dest_shared, target)
          "rm -rf #{dest} && cp -R #{target} #{dest}"
        }.join(" && ")
        logger.info(cmd) if logger
        run(cmd)
      end
    end
  end

end
